<?php

/**
 *
 */
class GratifyOrder extends ObjectModel
{
    public $id_gratify_orders;

    public $id_shop_default;

    public $id_order;

    public $id_cart;

    public $id_gratify_map_order;

    public $id_gratify_order;

    /**
     * @var string $date_add
     */
    public $date_add;

    /**
     * @var string $date_upd
     */
    public $date_upd;

    /**
     * @var array
     */
    public static $definition = array(
        'table' => 'gratify_orders',
        'primary' => 'id_gratify_orders',
        'multishop' => true,
        'fields' => array(
            'date_add' => array('type' => self::TYPE_DATE, 'validate' => 'isDate'),
            'date_upd' => array('type' => self::TYPE_DATE, 'validate' => 'isDate'),
            'id_shop_default' => array('type' => self::TYPE_INT, 'validate' => 'isInt'),

            'id_order' => array(
                'type' => self::TYPE_INT,
                'shop' => true,
                'validate' => 'isInt',
                'size' => 256
            ),
            'id_cart' => array(
                'type' => self::TYPE_INT,
                'shop' => true,
                'validate' => 'isInt',
                'size' => 256
            ),
            'id_gratify_map_order' => array(
                'type' => self::TYPE_STRING,
                'shop' => true,
                'validate' => 'isString',
                'size' => 256
            ),
            'id_gratify_order' => array(
                'type' => self::TYPE_STRING,
                'shop' => true,
                'validate' => 'isString',
                'size' => 256
            ),
        ),
    );

    /**
     * PrsProducts constructor.
     * @param null $id
     * @param null $id_lang
     * @param null $id_shop
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public function __construct($id = null, $id_lang = null, $id_shop = null)
    {
        Shop::addTableAssociation(self::$definition['table'], array('type' => 'shop'));
        parent::__construct($id, $id_lang, $id_shop);
    }

    /**
     * @param bool $auto_date
     * @param bool $null_values
     * @return bool
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public function add($auto_date = true, $null_values = false)
    {
        $context = Context::getContext();
        $this->id_shop_default = $context->shop->id;

        return parent::add($auto_date, $null_values);
    }

    /**
     * @param bool $null_values
     * @return bool
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
             */
    public function update($null_values = false)
    {
        $context = Context::getContext();
        $this->id_shop_default = $context->shop->id;

        return parent::update($null_values);
    }

    /**
     * @param $id_order
     * @return GratifyOrder|false
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public static function getByOrderId($id_order)
    {
        $shop_id = Context::getContext()->shop->id;

        $id = Db::getInstance()->getValue(
            ' SELECT go.' . self::$definition['primary'] . ' FROM '
            . _DB_PREFIX_ . static::$definition['table'] . ' AS go'
            . ' LEFT JOIN ' . _DB_PREFIX_ . self::$definition['table'] . '_shop AS gos '
            . ' ON (gos.' . self::$definition['primary'] . ' = go.' . self::$definition['primary'] . ' '
            . ' AND gos.`id_shop` = ' . (int)$shop_id . ')'
            . ' WHERE gos.id_order = ' . (int)$id_order
        );

        $gratifyOrder = new GratifyOrder($id, null, $shop_id);

        return Validate::isLoadedObject($gratifyOrder) ? $gratifyOrder : false;
    }

    /**
     * @param $id_gratify_order
     * @return GratifyOrder|false
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public static function getByGratifyOrderId($id_gratify_order)
    {
        $shop_id = Context::getContext()->shop->id;

        $id = Db::getInstance()->getValue(
            ' SELECT go.' . self::$definition['primary'] . ' FROM '
            . _DB_PREFIX_ . static::$definition['table'] . ' AS go'
            . ' LEFT JOIN ' . _DB_PREFIX_ . self::$definition['table'] . '_shop AS gos '
            . ' ON (gos.' . self::$definition['primary'] . ' = go.' . self::$definition['primary'] . ' '
            . ' AND gos.`id_shop` = ' . (int)$shop_id . ')'
            . ' WHERE gos.id_gratify_order = "' . pSQL($id_gratify_order) . '"'
        );

        $gratifyOrder = new GratifyOrder($id, null, $shop_id);

        return Validate::isLoadedObject($gratifyOrder) ? $gratifyOrder : false;
    }

    /**
     * @param $id_cart
     * @param $shop_id
     * @return GratifyOrder|false
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public static function getByCartId($id_cart, $shop_id)
    {
        $id = Db::getInstance()->getValue(
            ' SELECT go.' . self::$definition['primary'] . ' FROM '
            . _DB_PREFIX_ . static::$definition['table'] . ' AS go'
            . ' LEFT JOIN ' . _DB_PREFIX_ . self::$definition['table'] . '_shop AS gos '
            . ' ON (gos.' . self::$definition['primary'] . ' = go.' . self::$definition['primary'] . ' '
            . ' AND gos.`id_shop` = ' . (int)$shop_id . ')'
            . ' WHERE gos.id_cart = ' . (int)$id_cart
        );

        $gratifyOrder = new GratifyOrder($id, null, $shop_id);

        return Validate::isLoadedObject($gratifyOrder) ? $gratifyOrder : false;
    }
}
