<?php
/**
* 2007-2021 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author   SoftBuild <dev@soft-build.by>
*  @copyright 2017-2022 SoftBuild
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

use Gratify\PhpSdk\Client as GratifyClient;
use Gratify\PhpSdk\Response\ConfirmOrder;

class Gratify_PrestashopCallbackModuleFrontController extends ModuleFrontController
{
    public function postProcess()
    {
        if (!Tools::isSubmit('cart_id') || !Tools::isSubmit('secure_key')) {
            return false;
        }

        $cart_id = Tools::getValue('cart_id');
        $secure_key = Tools::getValue('secure_key');
        $merchantReference = Tools::getValue('merchantReference');
        $gratifyOrderId = Tools::getValue('orderId');

        $cart = new Cart((int) $cart_id);
        $customer = new Customer((int) $cart->id_customer);

        /**
         * Since it's an example we are validating the order right here,
         * You should not do it this way in your own module.
         */
        $payment_status = Configuration::get('PS_OS_PAYMENT'); // Default value for a payment that succeed.
        $message = null; // You can add a comment directly into the order so the merchant will see it in the BO.

        $order_id = Order::getIdByCartId((int) $cart->id);
        if (empty($order_id) && ($secure_key == $customer->secure_key)) {

            $client = new GratifyClient(
                Configuration::get('GRATIFY_MERCHANT_ID'),
                Configuration::get('GRATIFY_SECRET_KEY'),
                Configuration::get('GRATIFY_LIVE_MODE'),
                AddressHelper::isUs($cart->id_address_delivery)
            );

            $confirm = $client->confirmationOrder($gratifyOrderId, $merchantReference, Gratify_Prestashop::STORE_PLATFORM);
            if ($confirm->getOrderStatus() == ConfirmOrder::ORDER_STATUS_CREATED) {
                $module_name = $this->module->displayName;
                $this->module->validateOrder(
                    $cart_id,
                    $payment_status,
                    $cart->getOrderTotal(),
                    $module_name,
                    $message,
                    [
                        'transaction_id' => $gratifyOrderId
                    ],
                    $cart->id_currency,
                    false,
                    $secure_key
                );

                if ($gratifyOrder = GratifyOrder::getByCartId($cart_id, $cart->id_shop)) {
                    $order_id = Order::getIdByCartId((int) $cart->id);
                    $gratifyOrder->id_order = $order_id;
                    $gratifyOrder->save();
                }
                http_response_code(200);
            } else {
                http_response_code(400);
            }
        } else {
            http_response_code(400);
        }

        exit;
    }
}
